@@ -16,8 +16,6 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -45,11 +43,18 @@ DBA_OPEN_FUNC(lmdb)
 	MDB_env *env;
 	MDB_txn *txn;
 	int rc, mode = 0644, flags = MDB_NOSUBDIR;
+	zend_long mapsize = 0;
 
 	if(info->argc > 0) {
-		convert_to_long_ex(&info->argv[0]);
-		mode = Z_LVAL(info->argv[0]);
+		mode = zval_get_long(&info->argv[0]);
 
+		if (info->argc > 1) {
+			mapsize = zval_get_long(&info->argv[1]);
+			if (mapsize < 0) {
+				*error = "mapsize must be greater than or equal to zero";
+				return FAILURE;
+			}
+		}
 		/* TODO implement handling of the additional flags. */
 	}
 
@@ -59,6 +64,14 @@ DBA_OPEN_FUNC(lmdb)
 		return FAILURE;
 	}
 
+	if (mapsize > 0) {
+		rc = mdb_env_set_mapsize(env, (size_t) mapsize);
+		if (rc) {
+			*error = mdb_strerror(rc);
+			return FAILURE;
+		}
+	}
+
 	rc = mdb_env_open(env, info->path, flags, mode);
 	if (rc) {
 		*error = mdb_strerror(rc);
