@@ -53,7 +53,9 @@ const mbfl_encoding mbfl_encoding_sjis2004 = {
 	"Shift_JIS",
 	(const char *(*)[])&mbfl_encoding_sjis2004_aliases,
 	mblen_table_sjis,
-	MBFL_ENCTYPE_MBCS | MBFL_ENCTYPE_GL_UNSAFE
+	MBFL_ENCTYPE_MBCS | MBFL_ENCTYPE_GL_UNSAFE,
+	&vtbl_sjis2004_wchar,
+	&vtbl_wchar_sjis2004
 };
 
 const struct mbfl_identify_vtbl vtbl_identify_sjis2004 = {
