@@ -12,10 +12,10 @@
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
    +----------------------------------------------------------------------+
-   | Authors: Andi Gutmans <andi@zend.com>                                |
-   |          Zeev Suraski <zeev@zend.com>                                |
+   | Authors: Andi Gutmans <andi@php.net>                                 |
+   |          Zeev Suraski <zeev@php.net>                                 |
    |          Stanislav Malyshev <stas@zend.com>                          |
-   |          Dmitry Stogov <dmitry@zend.com>                             |
+   |          Dmitry Stogov <dmitry@php.net>                              |
    +----------------------------------------------------------------------+
 */
 
@@ -39,9 +39,6 @@
 		} else if (!IS_ACCEL_INTERNED(str)) { \
 			zend_string *tmp = accel_new_interned_string(str); \
 			if (tmp != (str)) { \
-				if (do_free) { \
-					/*zend_string_release(str);*/ \
-				} \
 				(str) = tmp; \
 			} else { \
 				ADD_STRING(str); \
@@ -56,19 +53,19 @@ static void zend_hash_persist_calc(HashTable *ht, void (*pPersistElement)(zval *
 	uint32_t idx;
 	Bucket *p;
 
-	if (!(ht->u.flags & HASH_FLAG_INITIALIZED) || ht->nNumUsed == 0) {
+	if (!(HT_FLAGS(ht) & HASH_FLAG_INITIALIZED) || ht->nNumUsed == 0) {
 		return;
 	}
 
-	if (!(ht->u.flags & HASH_FLAG_PACKED) && ht->nNumUsed < (uint32_t)(-(int32_t)ht->nTableMask) / 2) {
+	if (!(HT_FLAGS(ht) & HASH_FLAG_PACKED) && ht->nNumUsed < (uint32_t)(-(int32_t)ht->nTableMask) / 4) {
 		/* compact table */
 		uint32_t hash_size;
 
 		if (ht->nNumUsed <= HT_MIN_SIZE) {
-			hash_size = HT_MIN_SIZE;
+			hash_size = HT_MIN_SIZE * 2;
 		} else {
 			hash_size = (uint32_t)(-(int32_t)ht->nTableMask);
-			while (hash_size >> 1 > ht->nNumUsed) {
+			while (hash_size >> 2 > ht->nNumUsed) {
 				hash_size >>= 1;
 			}
 		}
@@ -94,9 +91,9 @@ static void zend_persist_ast_calc(zend_ast *ast)
 {
 	uint32_t i;
 
-	if (ast->kind == ZEND_AST_ZVAL) {
+	if (ast->kind == ZEND_AST_ZVAL || ast->kind == ZEND_AST_CONSTANT) {
 		ADD_SIZE(sizeof(zend_ast_zval));
-		zend_persist_zval_calc(zend_ast_get_zval(ast));
+		zend_persist_zval_calc(&((zend_ast_zval*)(ast))->val);
 	} else if (zend_ast_is_list(ast)) {
 		zend_ast_list *list = zend_ast_get_list(ast);
 		ADD_SIZE(sizeof(zend_ast_list) - sizeof(zend_ast *) + sizeof(zend_ast *) * list->children);
@@ -122,10 +119,9 @@ static void zend_persist_zval_calc(zval *z)
 
 	switch (Z_TYPE_P(z)) {
 		case IS_STRING:
-		case IS_CONSTANT:
 			ADD_INTERNED_STRING(Z_STR_P(z), 0);
 			if (ZSTR_IS_INTERNED(Z_STR_P(z))) {
-				Z_TYPE_FLAGS_P(z) &= ~ (IS_TYPE_REFCOUNTED | IS_TYPE_COPYABLE);
+				Z_TYPE_FLAGS_P(z) = 0;
 			}
 			break;
 		case IS_ARRAY:
@@ -154,10 +150,6 @@ static void zend_persist_zval_calc(zval *z)
 
 static void zend_persist_op_array_calc_ex(zend_op_array *op_array)
 {
-	if (op_array->type != ZEND_USER_FUNCTION) {
-		return;
-	}
-
 	if (op_array->static_variables) {
 		if (!zend_shared_alloc_get_xlat_entry(op_array->static_variables)) {
 			HashTable *old = op_array->static_variables;
@@ -263,19 +255,24 @@ static void zend_persist_op_array_calc(zval *zv)
 {
 	zend_op_array *op_array = Z_PTR_P(zv);
 
-	if (op_array->type == ZEND_USER_FUNCTION/* &&
-	    (!op_array->refcount || *(op_array->refcount) > 1)*/) {
-		zend_op_array *old_op_array = zend_shared_alloc_get_xlat_entry(op_array);
-		if (old_op_array) {
-			Z_PTR_P(zv) = old_op_array;
-		} else {
-			ADD_ARENA_SIZE(sizeof(zend_op_array));
-			zend_persist_op_array_calc_ex(Z_PTR_P(zv));
-			zend_shared_alloc_register_xlat_entry(op_array, Z_PTR_P(zv));
-		}
+	ZEND_ASSERT(op_array->type == ZEND_USER_FUNCTION);
+	ADD_SIZE(sizeof(zend_op_array));
+	zend_persist_op_array_calc_ex(Z_PTR_P(zv));
+}
+
+static void zend_persist_class_method_calc(zval *zv)
+{
+	zend_op_array *op_array = Z_PTR_P(zv);
+	zend_op_array *old_op_array;
+
+	ZEND_ASSERT(op_array->type == ZEND_USER_FUNCTION);
+	old_op_array = zend_shared_alloc_get_xlat_entry(op_array);
+	if (old_op_array) {
+		Z_PTR_P(zv) = old_op_array;
 	} else {
 		ADD_ARENA_SIZE(sizeof(zend_op_array));
 		zend_persist_op_array_calc_ex(Z_PTR_P(zv));
+		zend_shared_alloc_register_xlat_entry(op_array, Z_PTR_P(zv));
 	}
 }
 
@@ -315,7 +312,7 @@ static void zend_persist_class_entry_calc(zval *zv)
 	if (ce->type == ZEND_USER_CLASS) {
 		ADD_ARENA_SIZE(sizeof(zend_class_entry));
 		ADD_INTERNED_STRING(ce->name, 0);
-		zend_hash_persist_calc(&ce->function_table, zend_persist_op_array_calc);
+		zend_hash_persist_calc(&ce->function_table, zend_persist_class_method_calc);
 		if (ce->default_properties_table) {
 		    int i;
 
@@ -329,7 +326,9 @@ static void zend_persist_class_entry_calc(zval *zv)
 
 			ADD_SIZE(sizeof(zval) * ce->default_static_members_count);
 			for (i = 0; i < ce->default_static_members_count; i++) {
-				zend_persist_zval_calc(&ce->default_static_members_table[i]);
+				if (Z_TYPE(ce->default_static_members_table[i]) != IS_INDIRECT) {
+					zend_persist_zval_calc(&ce->default_static_members_table[i]);
+				}
 			}
 		}
 		zend_hash_persist_calc(&ce->constants_table, zend_persist_class_constant_calc);
@@ -346,14 +345,11 @@ static void zend_persist_class_entry_calc(zval *zv)
 		if (ce->trait_aliases) {
 			int i = 0;
 			while (ce->trait_aliases[i]) {
-				if (ce->trait_aliases[i]->trait_method) {
-					if (ce->trait_aliases[i]->trait_method->method_name) {
-						ADD_INTERNED_STRING(ce->trait_aliases[i]->trait_method->method_name, 0);
-					}
-					if (ce->trait_aliases[i]->trait_method->class_name) {
-						ADD_INTERNED_STRING(ce->trait_aliases[i]->trait_method->class_name, 0);
-					}
-					ADD_SIZE(sizeof(zend_trait_method_reference));
+				if (ce->trait_aliases[i]->trait_method.method_name) {
+					ADD_INTERNED_STRING(ce->trait_aliases[i]->trait_method.method_name, 0);
+				}
+				if (ce->trait_aliases[i]->trait_method.class_name) {
+					ADD_INTERNED_STRING(ce->trait_aliases[i]->trait_method.class_name, 0);
 				}
 
 				if (ce->trait_aliases[i]->alias) {
@@ -367,22 +363,16 @@ static void zend_persist_class_entry_calc(zval *zv)
 
 		if (ce->trait_precedences) {
 			int i = 0;
+			int j;
 
 			while (ce->trait_precedences[i]) {
-				ADD_INTERNED_STRING(ce->trait_precedences[i]->trait_method->method_name, 0);
-				ADD_INTERNED_STRING(ce->trait_precedences[i]->trait_method->class_name, 0);
-				ADD_SIZE(sizeof(zend_trait_method_reference));
+				ADD_INTERNED_STRING(ce->trait_precedences[i]->trait_method.method_name, 0);
+				ADD_INTERNED_STRING(ce->trait_precedences[i]->trait_method.class_name, 0);
 
-				if (ce->trait_precedences[i]->exclude_from_classes) {
-					int j = 0;
-
-					while (ce->trait_precedences[i]->exclude_from_classes[j].class_name) {
-						ADD_INTERNED_STRING(ce->trait_precedences[i]->exclude_from_classes[j].class_name, 0);
-						j++;
-					}
-					ADD_SIZE(sizeof(zend_class_entry*) * (j + 1));
+				for (j = 0; j < ce->trait_precedences[i]->num_excludes; j++) {
+					ADD_INTERNED_STRING(ce->trait_precedences[i]->exclude_class_names[j], 0);
 				}
-				ADD_SIZE(sizeof(zend_trait_precedence));
+				ADD_SIZE(sizeof(zend_trait_precedence) + (ce->trait_precedences[i]->num_excludes - 1) * sizeof(zend_string*));
 				i++;
 			}
 			ADD_SIZE(sizeof(zend_trait_precedence*) * (i + 1));
@@ -395,7 +385,7 @@ static void zend_accel_persist_class_table_calc(HashTable *class_table)
 	zend_hash_persist_calc(class_table, zend_persist_class_entry_calc);
 }
 
-uint32_t zend_accel_script_persist_calc(zend_persistent_script *new_persistent_script, char *key, unsigned int key_length, int for_shm)
+uint32_t zend_accel_script_persist_calc(zend_persistent_script *new_persistent_script, const char *key, unsigned int key_length, int for_shm)
 {
 	new_persistent_script->mem = NULL;
 	new_persistent_script->size = 0;
@@ -415,16 +405,22 @@ uint32_t zend_accel_script_persist_calc(zend_persistent_script *new_persistent_s
 	}
 	ADD_STRING(new_persistent_script->script.filename);
 
-#ifdef __SSE2__
+#if defined(__AVX__) || defined(__SSE2__)
 	/* Align size to 64-byte boundary */
 	new_persistent_script->size = (new_persistent_script->size + 63) & ~63;
 #endif
 
+	if (new_persistent_script->script.class_table.nNumUsed != new_persistent_script->script.class_table.nNumOfElements) {
+		zend_hash_rehash(&new_persistent_script->script.class_table);
+	}
 	zend_accel_persist_class_table_calc(&new_persistent_script->script.class_table);
+	if (new_persistent_script->script.function_table.nNumUsed != new_persistent_script->script.function_table.nNumOfElements) {
+		zend_hash_rehash(&new_persistent_script->script.function_table);
+	}
 	zend_hash_persist_calc(&new_persistent_script->script.function_table, zend_persist_op_array_calc);
 	zend_persist_op_array_calc_ex(&new_persistent_script->script.main_op_array);
 
-#ifdef __SSE2__
+#if defined(__AVX__) || defined(__SSE2__)
 	/* Align size to 64-byte boundary */
 	new_persistent_script->arena_size = (new_persistent_script->arena_size + 63) & ~63;
 #endif
