@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2017 The PHP Group                                |
+  | Copyright (c) 1997-2018 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,8 +16,6 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -263,7 +261,7 @@ static sb4 oci_bind_output_cb(dvoid *ctx, OCIBind *bindp, ub4 iter, ub4 index, d
 	}
 
 	convert_to_string(parameter);
-	zval_dtor(parameter);
+	zval_ptr_dtor_str(parameter);
 
 	Z_STR_P(parameter) = zend_string_alloc(param->max_value_len, 1);
 	P->used_for_output = 1;
@@ -381,7 +379,7 @@ static int oci_stmt_param_hook(pdo_stmt_t *stmt, struct pdo_bound_param_data *pa
 							/* OCI likes to stick non-terminated strings in things */
 							*Z_STRVAL_P(parameter) = '\0';
 						}
-						zval_dtor(parameter);
+						zval_ptr_dtor_str(parameter);
 						ZVAL_UNDEF(parameter);
 					} else if (Z_TYPE_P(parameter) == IS_STRING) {
 						Z_STR_P(parameter) = zend_string_init(Z_STRVAL_P(parameter), P->actual_len, 1);
@@ -599,12 +597,12 @@ static int oci_stmt_describe(pdo_stmt_t *stmt, int colno) /* {{{ */
 			} else if (dtype == SQLT_IBFLOAT || dtype == SQLT_IBDOUBLE) {
 				S->cols[colno].datalen = 1024;
 #endif
+			} else if (dtype == SQLT_BIN) {
+				S->cols[colno].datalen = (ub4) col->maxlen * 2; // raw characters to hex digits
 			} else {
-				S->cols[colno].datalen = (ub4) col->maxlen;
-			}
-			if (dtype == SQLT_BIN) {
-				S->cols[colno].datalen *= 3;
+				S->cols[colno].datalen = (ub4) (col->maxlen * S->H->max_char_width);
 			}
+
 			S->cols[colno].data = emalloc(S->cols[colno].datalen + 1);
 			dtype = SQLT_CHR;
 
@@ -691,7 +689,7 @@ static int oci_blob_close(php_stream *stream, int close_handle)
 
 		OCILobClose(self->E->svc, self->E->err, self->lob);
 		zval_ptr_dtor(&self->dbh);
-		GC_REFCOUNT(obj)--;
+		GC_DELREF(obj);
 		efree(self->E);
 		efree(self);
 	}
@@ -719,7 +717,7 @@ static int oci_blob_seek(php_stream *stream, zend_off_t offset, int whence, zend
 	}
 }
 
-static php_stream_ops oci_blob_stream_ops = {
+static const php_stream_ops oci_blob_stream_ops = {
 	oci_blob_write,
 	oci_blob_read,
 	oci_blob_close,
@@ -751,7 +749,7 @@ static php_stream *oci_create_lob_stream(zval *dbh, pdo_stmt_t *stmt, OCILobLoca
 		zend_object *obj;
 		obj = &stmt->std;
 		Z_ADDREF(self->dbh);
-		GC_REFCOUNT(obj)++;
+		GC_ADDREF(obj);
 		return stm;
 	}
 
@@ -795,7 +793,7 @@ static int oci_stmt_get_col(pdo_stmt_t *stmt, int colno, char **ptr, size_t *len
 	}
 } /* }}} */
 
-struct pdo_stmt_methods oci_stmt_methods = {
+const struct pdo_stmt_methods oci_stmt_methods = {
 	oci_stmt_dtor,
 	oci_stmt_execute,
 	oci_stmt_fetch,
