@@ -1,8 +1,8 @@
 /*
   +----------------------------------------------------------------------+
-  | PHP Version 5                                                        |
+  | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2016 The PHP Group                                |
+  | Copyright (c) 1997-2018 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -14,28 +14,25 @@
   +----------------------------------------------------------------------+
   | Authors: Brad Lafountain <rodif_bl@yahoo.com>                        |
   |          Shane Caraveo <shane@caraveo.com>                           |
-  |          Dmitry Stogov <dmitry@zend.com>                             |
+  |          Dmitry Stogov <dmitry@php.net>                              |
   +----------------------------------------------------------------------+
 */
-/* $Id$ */
 
 #ifndef PHP_HTTP_H
 #define PHP_HTTP_H
 
-int make_http_soap_request(zval  *this_ptr, 
-                           char  *request, 
-                           int    request_size, 
-                           char  *location, 
-                           char  *soapaction, 
-                           int    soap_version,
-                           char **response, 
-                           int   *response_len TSRMLS_DC);
+int make_http_soap_request(zval        *this_ptr,
+                           zend_string *request,
+                           char        *location,
+                           char        *soapaction,
+                           int          soap_version,
+                           zval        *response);
 
-int proxy_authentication(zval* this_ptr, smart_str* soap_headers TSRMLS_DC);
-int basic_authentication(zval* this_ptr, smart_str* soap_headers TSRMLS_DC);
+int proxy_authentication(zval* this_ptr, smart_str* soap_headers);
+int basic_authentication(zval* this_ptr, smart_str* soap_headers);
 void http_context_headers(php_stream_context* context,
                           zend_bool has_authorization,
                           zend_bool has_proxy_authorization,
                           zend_bool has_cookies,
-                          smart_str* soap_headers TSRMLS_DC);
+                          smart_str* soap_headers);
 #endif
