@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2008-2016 The PHP Group                                |
+   | Copyright (c) 2008-2018 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -44,13 +44,13 @@
 PHP_FUNCTION(dns_get_mx) /* {{{ */
 {
 	char *hostname;
-	int hostname_len;
+	size_t hostname_len;
 	zval *mx_list, *weight_list = NULL;
 
 	DNS_STATUS      status;                 /* Return value of DnsQuery_A() function */
 	PDNS_RECORD     pResult, pRec;          /* Pointer to DNS_RECORD structure */
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sz|z", &hostname, &hostname_len, &mx_list, &weight_list) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sz/|z/", &hostname, &hostname_len, &mx_list, &weight_list) == FAILURE) {
 		return;
 	}
 
@@ -60,11 +60,11 @@ PHP_FUNCTION(dns_get_mx) /* {{{ */
 		RETURN_FALSE;
 	}
 
-	zval_dtor(mx_list);
+	zval_ptr_dtor(mx_list);
 	array_init(mx_list);
 
 	if (weight_list) {
-		zval_dtor(weight_list);
+		zval_ptr_dtor(weight_list);
 		array_init(weight_list);
 	}
 
@@ -75,7 +75,7 @@ PHP_FUNCTION(dns_get_mx) /* {{{ */
 			continue;
 		}
 
-		add_next_index_string(mx_list, pRec->Data.MX.pNameExchange, 1);
+		add_next_index_string(mx_list, pRec->Data.MX.pNameExchange);
 		if (weight_list) {
 			add_next_index_long(weight_list, srv->wPriority);
 		}
@@ -93,18 +93,18 @@ PHP_FUNCTION(dns_get_mx) /* {{{ */
 PHP_FUNCTION(dns_check_record)
 {
 	char *hostname, *rectype = NULL;
-	int hostname_len, rectype_len = 0;
+	size_t hostname_len, rectype_len = 0;
 	int type = DNS_TYPE_MX;
 
 	DNS_STATUS      status;                 /* Return value of DnsQuery_A() function */
 	PDNS_RECORD     pResult;          /* Pointer to DNS_RECORD structure */
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|s", &hostname, &hostname_len, &rectype, &rectype_len) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|s", &hostname, &hostname_len, &rectype, &rectype_len) == FAILURE) {
 		return;
 	}
 
 	if (hostname_len == 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Host cannot be empty");
+		php_error_docref(NULL, E_WARNING, "Host cannot be empty");
 		RETURN_FALSE;
 	}
 
@@ -122,7 +122,7 @@ PHP_FUNCTION(dns_check_record)
 		else if (!strcasecmp("NAPTR", rectype)) type = DNS_TYPE_NAPTR;
 		else if (!strcasecmp("A6",    rectype)) type = DNS_TYPE_A6;
 		else {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Type '%s' not supported", rectype);
+			php_error_docref(NULL, E_WARNING, "Type '%s' not supported", rectype);
 			RETURN_FALSE;
 		}
 	}
@@ -138,7 +138,7 @@ PHP_FUNCTION(dns_check_record)
 /* }}} */
 
 /* {{{ php_parserr */
-static void php_parserr(PDNS_RECORD pRec, int type_to_fetch, int store, int raw, zval **subarray)
+static void php_parserr(PDNS_RECORD pRec, int type_to_fetch, int store, int raw, zval *subarray)
 {
 	int type;
 	u_long ttl;
@@ -146,6 +146,8 @@ static void php_parserr(PDNS_RECORD pRec, int type_to_fetch, int store, int raw,
 	type = pRec->wType;
 	ttl = pRec->dwTtl;
 
+	ZVAL_UNDEF(subarray);
+
 	if (type_to_fetch != DNS_TYPE_ANY && type != type_to_fetch) {
 		return;
 	}
@@ -154,16 +156,15 @@ static void php_parserr(PDNS_RECORD pRec, int type_to_fetch, int store, int raw,
 		return;
 	}
 
-	ALLOC_INIT_ZVAL(*subarray);
-	array_init(*subarray);
+	array_init(subarray);
 
-	add_assoc_string(*subarray, "host", pRec->pName, 1);
-	add_assoc_string(*subarray, "class", "IN", 1);
-	add_assoc_long(*subarray, "ttl", ttl);
+	add_assoc_string(subarray, "host", pRec->pName);
+	add_assoc_string(subarray, "class", "IN");
+	add_assoc_long(subarray, "ttl", ttl);
 
 	if (raw) {
-		add_assoc_long(*subarray, "type", type);
-		add_assoc_stringl(*subarray, "data", (char*) &pRec->Data, (uint) pRec->wDataLength, 1);
+		add_assoc_long(subarray, "type", type);
+		add_assoc_stringl(subarray, "data", (char*) &pRec->Data, (uint32_t) pRec->wDataLength);
 		return;
 	}
 
@@ -171,33 +172,33 @@ static void php_parserr(PDNS_RECORD pRec, int type_to_fetch, int store, int raw,
 		case DNS_TYPE_A: {
 			IN_ADDR ipaddr;
 			ipaddr.S_un.S_addr = (pRec->Data.A.IpAddress);
-			add_assoc_string(*subarray, "type", "A", 1);
-			add_assoc_string(*subarray, "ip", inet_ntoa(ipaddr), 1);
+			add_assoc_string(subarray, "type", "A");
+			add_assoc_string(subarray, "ip", inet_ntoa(ipaddr));
 			break;
 		}
 
 		case DNS_TYPE_MX:
-			add_assoc_string(*subarray, "type", "MX", 1);
-			add_assoc_long(*subarray, "pri", pRec->Data.Srv.wPriority);
+			add_assoc_string(subarray, "type", "MX");
+			add_assoc_long(subarray, "pri", pRec->Data.Srv.wPriority);
 			/* no break; */
 
 		case DNS_TYPE_CNAME:
 			if (type == DNS_TYPE_CNAME) {
-				add_assoc_string(*subarray, "type", "CNAME", 1);
+				add_assoc_string(subarray, "type", "CNAME");
 			}
 			/* no break; */
 
 		case DNS_TYPE_NS:
 			if (type == DNS_TYPE_NS) {
-				add_assoc_string(*subarray, "type", "NS", 1);
+				add_assoc_string(subarray, "type", "NS");
 			}
 			/* no break; */
 
 		case DNS_TYPE_PTR:
 			if (type == DNS_TYPE_PTR) {
-				add_assoc_string(*subarray, "type", "PTR", 1);
+				add_assoc_string(subarray, "type", "PTR");
 			}
-			add_assoc_string(*subarray, "target", pRec->Data.MX.pNameExchange, 1);
+			add_assoc_string(subarray, "target", pRec->Data.MX.pNameExchange);
 			break;
 
 		/* Not available on windows, the query is possible but there is no DNS_HINFO_DATA structure */
@@ -207,30 +208,30 @@ static void php_parserr(PDNS_RECORD pRec, int type_to_fetch, int store, int raw,
 				DWORD i = 0;
 				DNS_TXT_DATA *data_txt = &pRec->Data.TXT;
 				DWORD count = data_txt->dwStringCount;
-				char *txt, *txt_dst;
-				long txt_len = 0;
-				zval *entries;
+				zend_string *txt;
+				char *txt_dst;
+				size_t txt_len = 0;
+				zval entries;
+
+				add_assoc_string(subarray, "type", "TXT");
+
+				array_init(&entries);
 
-				add_assoc_string(*subarray, "type", "TXT", 1);
-				
-				ALLOC_INIT_ZVAL(entries);
-				array_init(entries);
-				
 				for (i = 0; i < count; i++) {
 					txt_len += strlen(data_txt->pStringArray[i]) + 1;
 				}
 
-				txt = ecalloc(txt_len * 2, 1);
-				txt_dst = txt;
+				txt = zend_string_safe_alloc(txt_len, 2, 0, 0);
+				txt_dst = txt->val;
 				for (i = 0; i < count; i++) {
-					int len = strlen(data_txt->pStringArray[i]);
+					size_t len = strlen(data_txt->pStringArray[i]);
 					memcpy(txt_dst, data_txt->pStringArray[i], len);
-					add_next_index_stringl(entries, data_txt->pStringArray[i], len, 1);
+					add_next_index_stringl(&entries, data_txt->pStringArray[i], len);
 					txt_dst += len;
 				}
-
-				add_assoc_string(*subarray, "txt", txt, 0);
-				add_assoc_zval(*subarray, "entries", entries);
+				txt->len = txt_dst - txt->val;
+				add_assoc_str(subarray, "txt", txt);
+				add_assoc_zval(subarray, "entries", &entries);
 			}
 			break;
 
@@ -238,15 +239,15 @@ static void php_parserr(PDNS_RECORD pRec, int type_to_fetch, int store, int raw,
 			{
 				DNS_SOA_DATA *data_soa = &pRec->Data.Soa;
 
-				add_assoc_string(*subarray, "type", "SOA", 1);
+				add_assoc_string(subarray, "type", "SOA");
 
-				add_assoc_string(*subarray, "mname", data_soa->pNamePrimaryServer, 1);
-				add_assoc_string(*subarray, "rname", data_soa->pNameAdministrator, 1);
-				add_assoc_long(*subarray, "serial", data_soa->dwSerialNo);
-				add_assoc_long(*subarray, "refresh", data_soa->dwRefresh);
-				add_assoc_long(*subarray, "retry", data_soa->dwRetry);
-				add_assoc_long(*subarray, "expire", data_soa->dwExpire);
-				add_assoc_long(*subarray, "minimum-ttl", data_soa->dwDefaultTtl);
+				add_assoc_string(subarray, "mname", data_soa->pNamePrimaryServer);
+				add_assoc_string(subarray, "rname", data_soa->pNameAdministrator);
+				add_assoc_long(subarray, "serial", data_soa->dwSerialNo);
+				add_assoc_long(subarray, "refresh", data_soa->dwRefresh);
+				add_assoc_long(subarray, "retry", data_soa->dwRetry);
+				add_assoc_long(subarray, "expire", data_soa->dwExpire);
+				add_assoc_long(subarray, "minimum-ttl", data_soa->dwDefaultTtl);
 			}
 			break;
 
@@ -294,8 +295,8 @@ static void php_parserr(PDNS_RECORD pRec, int type_to_fetch, int store, int raw,
 				}
 				tp[0] = '\0';
 
-				add_assoc_string(*subarray, "type", "AAAA", 1);
-				add_assoc_string(*subarray, "ipv6", buf, 1);
+				add_assoc_string(subarray, "type", "AAAA");
+				add_assoc_string(subarray, "ipv6", buf);
 			}
 			break;
 
@@ -309,73 +310,70 @@ static void php_parserr(PDNS_RECORD pRec, int type_to_fetch, int store, int raw,
 			{
 				DNS_SRV_DATA *data_srv = &pRec->Data.Srv;
 
-				add_assoc_string(*subarray, "type", "SRV", 1);
-				add_assoc_long(*subarray, "pri", data_srv->wPriority);
-				add_assoc_long(*subarray, "weight", data_srv->wWeight);
-				add_assoc_long(*subarray, "port", data_srv->wPort);
-				add_assoc_string(*subarray, "target", data_srv->pNameTarget, 1);
+				add_assoc_string(subarray, "type", "SRV");
+				add_assoc_long(subarray, "pri", data_srv->wPriority);
+				add_assoc_long(subarray, "weight", data_srv->wWeight);
+				add_assoc_long(subarray, "port", data_srv->wPort);
+				add_assoc_string(subarray, "target", data_srv->pNameTarget);
 			}
 			break;
 
-#if _MSC_VER >= 1500
 		case DNS_TYPE_NAPTR:
 			{
 				DNS_NAPTR_DATA * data_naptr = &pRec->Data.Naptr;
 
-				add_assoc_string(*subarray, "type", "NAPTR", 1);
-				add_assoc_long(*subarray, "order", data_naptr->wOrder);
-				add_assoc_long(*subarray, "pref", data_naptr->wPreference);
-				add_assoc_string(*subarray, "flags", data_naptr->pFlags, 1);
-				add_assoc_string(*subarray, "services", data_naptr->pService, 1);
-				add_assoc_string(*subarray, "regex", data_naptr->pRegularExpression, 1);
-				add_assoc_string(*subarray, "replacement", data_naptr->pReplacement, 1);
+				add_assoc_string(subarray, "type", "NAPTR");
+				add_assoc_long(subarray, "order", data_naptr->wOrder);
+				add_assoc_long(subarray, "pref", data_naptr->wPreference);
+				add_assoc_string(subarray, "flags", data_naptr->pFlags);
+				add_assoc_string(subarray, "services", data_naptr->pService);
+				add_assoc_string(subarray, "regex", data_naptr->pRegularExpression);
+				add_assoc_string(subarray, "replacement", data_naptr->pReplacement);
 			}
 			break;
-#endif
 
 		default:
 			/* unknown type */
-			zval_ptr_dtor(subarray);
-			*subarray = NULL;
+			ZVAL_UNDEF(subarray);
 			return;
 	}
 
 }
 /* }}} */
 
-/* {{{ proto array|false dns_get_record(string hostname [, int type[, array authns, array addtl]])
+/* {{{ proto array|false dns_get_record(string hostname [, int type[, array &authns[, array &addtl[, bool raw]]]])
    Get any Resource Record corresponding to a given Internet host name */
 PHP_FUNCTION(dns_get_record)
 {
 	char *hostname;
-	int hostname_len;
-	long type_param = PHP_DNS_ANY;
+	size_t hostname_len;
+	zend_long type_param = PHP_DNS_ANY;
 	zval *authns = NULL, *addtl = NULL;
 	int type, type_to_fetch, first_query = 1, store_results = 1;
 	zend_bool raw = 0;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|lz!z!b",
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|lz/!z/!b",
 			&hostname, &hostname_len, &type_param, &authns, &addtl, &raw) == FAILURE) {
 		return;
 	}
 
 	if (authns) {
-		zval_dtor(authns);
+		zval_ptr_dtor(authns);
 		array_init(authns);
 	}
 	if (addtl) {
-		zval_dtor(addtl);
+		zval_ptr_dtor(addtl);
 		array_init(addtl);
 	}
 
 	if (!raw) {
 		if ((type_param & ~PHP_DNS_ALL) && (type_param != PHP_DNS_ANY)) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Type '%ld' not supported", type_param);
+			php_error_docref(NULL, E_WARNING, "Type '%ld' not supported", type_param);
 			RETURN_FALSE;
 		}
 	} else {
 		if ((type_param < 1) || (type_param > 0xFFFF)) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING,
+			php_error_docref(NULL, E_WARNING,
 				"Numeric DNS record type must be between 1 and 65535, '%ld' given", type_param);
 			RETURN_FALSE;
 		}
@@ -458,28 +456,27 @@ PHP_FUNCTION(dns_get_record)
 				if (status == DNS_INFO_NO_RECORDS || status == DNS_ERROR_RCODE_NAME_ERROR) {
 					continue;
 				} else {
-					php_error_docref(NULL TSRMLS_CC, E_WARNING, "DNS Query failed");
-					zval_dtor(return_value);
+					php_error_docref(NULL, E_WARNING, "DNS Query failed");
+					zend_array_destroy(Z_ARR_P(return_value));
 					RETURN_FALSE;
 				}
 			}
 
 			for (pRec = pResult; pRec; pRec = pRec->pNext) {
-				DNS_SRV_DATA *srv = &pRec->Data.Srv;
-				zval *retval = NULL;
+				zval retval;
 
 				if (pRec->Flags.S.Section == DnsSectionAnswer) {
 					php_parserr(pRec, type_to_fetch, store_results, raw, &retval);
-					if (retval != NULL && store_results) {
-						add_next_index_zval(return_value, retval);
+					if (!Z_ISUNDEF(retval) && store_results) {
+						add_next_index_zval(return_value, &retval);
 					}
 				}
 
 				if (authns && pRec->Flags.S.Section == DnsSectionAuthority) {
 
 					php_parserr(pRec, type_to_fetch, 1, raw, &retval);
-					if (retval != NULL) {
-						add_next_index_zval(authns, retval);
+					if (!Z_ISUNDEF(retval)) {
+						add_next_index_zval(authns, &retval);
 					}
 				}
 
@@ -493,8 +490,8 @@ PHP_FUNCTION(dns_get_record)
 #endif
 				if (addtl && pRec->Flags.S.Section == DnsSectionAdditional) {
 					php_parserr(pRec, type_to_fetch, 1, raw, &retval);
-					if (retval != NULL) {
-						add_next_index_zval(addtl, retval);
+					if (!Z_ISUNDEF(retval)) {
+						add_next_index_zval(addtl, &retval);
 					}
 				}
 			}
@@ -504,3 +501,21 @@ PHP_FUNCTION(dns_get_record)
 	}
 }
 /* }}} */
+
+PHP_MINIT_FUNCTION(dns) {
+	REGISTER_LONG_CONSTANT("DNS_A",     PHP_DNS_A,     CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("DNS_NS",    PHP_DNS_NS,    CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("DNS_CNAME", PHP_DNS_CNAME, CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("DNS_SOA",   PHP_DNS_SOA,   CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("DNS_PTR",   PHP_DNS_PTR,   CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("DNS_HINFO", PHP_DNS_HINFO, CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("DNS_MX",    PHP_DNS_MX,    CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("DNS_TXT",   PHP_DNS_TXT,   CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("DNS_SRV",   PHP_DNS_SRV,   CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("DNS_NAPTR", PHP_DNS_NAPTR, CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("DNS_AAAA",  PHP_DNS_AAAA,  CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("DNS_A6",    PHP_DNS_A6,    CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("DNS_ANY",   PHP_DNS_ANY,   CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("DNS_ALL",   PHP_DNS_ALL,   CONST_CS | CONST_PERSISTENT);
+	return SUCCESS;
+}
