@@ -16,11 +16,11 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #ifndef PHP_RANDOM_H
 #define PHP_RANDOM_H
 
+BEGIN_EXTERN_C()
+
 PHP_FUNCTION(random_bytes);
 PHP_FUNCTION(random_int);
 
@@ -50,6 +50,8 @@ extern PHPAPI int random_globals_id;
 extern PHPAPI php_random_globals random_globals;
 #endif
 
+END_EXTERN_C()
+
 #endif
 
 /*
