@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2016 The PHP Group                                |
+   | Copyright (c) 1997-2018 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -27,7 +27,7 @@ typedef struct php_cli_server_ext_mime_type_pair {
 	const char *mime_type;
 } php_cli_server_ext_mime_type_pair;
 
-static php_cli_server_ext_mime_type_pair mime_type_map[] = {
+static const php_cli_server_ext_mime_type_pair mime_type_map[] = {
 	{ "ez", "application/andrew-inset" },
 	{ "aw", "application/applixware" },
 	{ "atom", "application/atom+xml" },
@@ -635,16 +635,12 @@ static php_cli_server_ext_mime_type_pair mime_type_map[] = {
 	{ "bdf", "application/x-font-bdf" },
 	{ "gsf", "application/x-font-ghostscript" },
 	{ "psf", "application/x-font-linux-psf" },
-	{ "otf", "application/x-font-otf" },
 	{ "pcf", "application/x-font-pcf" },
 	{ "snf", "application/x-font-snf" },
-	{ "ttf", "application/x-font-ttf" },
-	{ "ttc", "application/x-font-ttf" },
 	{ "pfa", "application/x-font-type1" },
 	{ "pfb", "application/x-font-type1" },
 	{ "pfm", "application/x-font-type1" },
 	{ "afm", "application/x-font-type1" },
-	{ "woff", "application/font-woff" },
 	{ "arc", "application/x-freearc" },
 	{ "spl", "application/x-futuresplash" },
 	{ "gca", "application/x-gca-compressed" },
@@ -758,6 +754,7 @@ static php_cli_server_ext_mime_type_pair mime_type_map[] = {
 	{ "midi", "audio/midi" },
 	{ "kar", "audio/midi" },
 	{ "rmi", "audio/midi" },
+	{ "m4a", "audio/mp4" },
 	{ "mp4a", "audio/mp4" },
 	{ "mpga", "audio/mpeg" },
 	{ "mp2", "audio/mpeg" },
@@ -804,6 +801,11 @@ static php_cli_server_ext_mime_type_pair mime_type_map[] = {
 	{ "cml", "chemical/x-cml" },
 	{ "csml", "chemical/x-csml" },
 	{ "xyz", "chemical/x-xyz" },
+	{ "ttc", "font/collection" },
+	{ "otf", "font/otf" },
+	{ "ttf", "font/ttf" },
+	{ "woff", "font/woff" },
+	{ "woff2", "font/woff2" },
 	{ "bmp", "image/bmp" },
 	{ "cgm", "image/cgm" },
 	{ "g3", "image/g3fax" },
@@ -825,9 +827,9 @@ static php_cli_server_ext_mime_type_pair mime_type_map[] = {
 	{ "uvvi", "image/vnd.dece.graphic" },
 	{ "uvg", "image/vnd.dece.graphic" },
 	{ "uvvg", "image/vnd.dece.graphic" },
-	{ "sub", "text/vnd.dvb.subtitle" },
 	{ "djvu", "image/vnd.djvu" },
 	{ "djv", "image/vnd.djvu" },
+	{ "sub", "text/vnd.dvb.subtitle" },
 	{ "dwg", "image/vnd.dwg" },
 	{ "dxf", "image/vnd.dxf" },
 	{ "fbs", "image/vnd.fastbidsheet" },
@@ -917,8 +919,8 @@ static php_cli_server_ext_mime_type_pair mime_type_map[] = {
 	{ "vcard", "text/vcard" },
 	{ "curl", "text/vnd.curl" },
 	{ "dcurl", "text/vnd.curl.dcurl" },
-	{ "scurl", "text/vnd.curl.scurl" },
 	{ "mcurl", "text/vnd.curl.mcurl" },
+	{ "scurl", "text/vnd.curl.scurl" },
 	{ "fly", "text/vnd.fly" },
 	{ "flx", "text/vnd.fmi.flexstor" },
 	{ "gv", "text/vnd.graphviz" },
@@ -941,10 +943,10 @@ static php_cli_server_ext_mime_type_pair mime_type_map[] = {
 	{ "f77", "text/x-fortran" },
 	{ "f90", "text/x-fortran" },
 	{ "java", "text/x-java-source" },
+	{ "nfo", "text/x-nfo" },
 	{ "opml", "text/x-opml" },
 	{ "p", "text/x-pascal" },
 	{ "pas", "text/x-pascal" },
-	{ "nfo", "text/x-nfo" },
 	{ "etx", "text/x-setext" },
 	{ "sfv", "text/x-sfv" },
 	{ "uu", "text/x-uuencode" },
@@ -1010,6 +1012,7 @@ static php_cli_server_ext_mime_type_pair mime_type_map[] = {
 	{ "smv", "video/x-smv" },
 	{ "ice", "x-conference/x-cooltalk" },
 	{ "map", "application/json" },
+	{ "jsm", "application/javascript" },
 	{ NULL, NULL }
 };
 
