@@ -1,7 +1,7 @@
-/*
+﻿/*
  *    PHP Sendmail for Windows.
  *
- *  This file is rewriten specificly for PHPFI.  Some functionality
+ *  This file is rewritten specificly for PHPFI.  Some functionality
  *  has been removed (MIME and file attachments).  This code was
  *  modified from code based on code written by Jarle Aase.
  *
@@ -17,8 +17,6 @@
  *
  */
 
-/* $Id$ */
-
 #include "php.h"				/*php specific */
 #include <stdio.h>
 #include <stdlib.h>
@@ -36,10 +34,7 @@
 
 #include "php_win32_globals.h"
 
-#if HAVE_PCRE || HAVE_BUNDLED_PCRE
 #include "ext/pcre/php_pcre.h"
-#endif
-
 #include "ext/standard/php_string.h"
 #include "ext/date/php_date.h"
 
@@ -130,9 +125,6 @@ static char *ErrorMessages[] =
  */
 static zend_string *php_win32_mail_trim_header(char *header)
 {
-
-#if HAVE_PCRE || HAVE_BUNDLED_PCRE
-
 	zend_string *result, *result2;
 	zend_string *replace;
 	zend_string *regex;
@@ -145,13 +137,13 @@ static zend_string *php_win32_mail_trim_header(char *header)
 	regex = zend_string_init(PHP_WIN32_MAIL_UNIFY_PATTERN, sizeof(PHP_WIN32_MAIL_UNIFY_PATTERN)-1, 0);
 
 	result = php_pcre_replace(regex,
-				  NULL, header, (int)strlen(header),
+				  NULL, header, strlen(header),
 				  replace,
 				  -1,
 				  NULL);
 
-	zend_string_release(replace);
-	zend_string_release(regex);
+	zend_string_release_ex(replace, 0);
+	zend_string_release_ex(regex, 0);
 
 	if (NULL == result) {
 		return NULL;
@@ -161,19 +153,15 @@ static zend_string *php_win32_mail_trim_header(char *header)
 	regex = zend_string_init(PHP_WIN32_MAIL_RMVDBL_PATTERN, sizeof(PHP_WIN32_MAIL_RMVDBL_PATTERN)-1, 0);
 
 	result2 = php_pcre_replace(regex,
-				   result, ZSTR_VAL(result), (int)ZSTR_LEN(result),
+				   result, ZSTR_VAL(result), ZSTR_LEN(result),
 				   replace,
 				  -1,
 				  NULL);
-	zend_string_release(replace);
-	zend_string_release(regex);
-	zend_string_release(result);
+	zend_string_release_ex(replace, 0);
+	zend_string_release_ex(regex, 0);
+	zend_string_release_ex(result, 0);
 
 	return result2;
-#else
-	/* In case we don't have PCRE support (for whatever reason...) simply do nothing and return the unmodified header */
-	return zend_string_init(header, strlen(header), 0);
-#endif
 }
 
 /*********************************************************************
@@ -346,9 +334,6 @@ PHPAPI char *GetSMErrorText(int index)
 	}
 }
 
-PHPAPI zend_string *php_str_to_str(char *haystack, size_t length, char *needle,
-		size_t needle_len, char *str, size_t str_len);
-
 
 /*********************************************************************
 // Name:  SendText
